#!/bin/sh                                                                                                     
# This file is part of Download Manager.
#                                                    
# Download Manager is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Download Manager is distributed in the hope that it will be useful,                             
# but WITHOUT ANY WARRANTY; without even the implied warranty of           
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#                                         
# You should have received a copy of the GNU General Public License          
# along with Download Manager. If not, see <http://www.gnu.org/licenses/>.

# Load the common functions
. _scripts/lib.sh

speed_limit=$(selectDBValue preferences value variable speed_limit)
max_jobs=$(selectDBValue preferences value variable max_jobs)
no_retries=$(selectDBValue preferences value variable no_retries)

if [ "$speed_limit" != "0" ];then
    speed_limit=$(echo ${speed_limit}K)
fi

#Change the directory to $homedir
cd $homedir

selectDBLinksDownloading()
{
    DB_Locked=1
    while [ $DB_Locked -ne 0 ];do
        Output=$(${SQLITE} ${DBFNAME} "SELECT link FROM links WHERE status=\"Downloading\" OR status=\"Loading\" OR status=\"Connecting\" ORDER BY sno" 2>&1)
        DB_Locked=$(echo $Output|grep "database is locked"|wc -l)
    done
    echo "$Output"
}

selectDBNumLinksActive()
{
    DB_Locked=1
    while [ $DB_Locked -ne 0 ];do
        Output=$(${SQLITE} ${DBFNAME} "SELECT count(*) FROM links WHERE status=\"Downloading\" OR status=\"Loading\" OR status=\"Connecting\"" 2>&1)
        DB_Locked=$(echo $Output|grep "database is locked"|wc -l)
    done
    echo "$Output"
}

selectDBLinksQuery()
{
    DB_Locked=1
    while [ $DB_Locked -ne 0 ];do
        Output=$(${SQLITE} ${DBFNAME} "SELECT link FROM links WHERE ${1} ORDER BY sno" 2>&1)
        DB_Locked=$(echo $Output|grep "database is locked"|wc -l)
    done
    echo "$Output"
}

startAria2c()
{
    ${ARIA2C} --enable-rpc --max-overall-download-limit=${speed_limit} --max-concurrent-downloads=${max_jobs} \
        --auto-save-interval=1 --max-tries=${no_retries} --use-head=false --log-level=info --event-poll=select \
        --quiet=true --disable-ipv6=true & echo $! > _tmp/.pid_aria2c
}

if [ -f "_tmp/.pid_aria2c" ];then
    PID=$(cat _tmp/.pid_aria2c)
    if [ ! -d /proc/${PID} ];then
        startAria2c
    fi
else
    startAria2c
fi

#First time start. Change all Downloading links to Waiting
URL=$(selectDBLinksDownloading)
for url in $(echo "$URL")
do
    GID=$(selectDBValue links gid link "$url")
    LinkStatus=$(${XMLRPC} ${SERVER_URL} aria2.tellStatus ${GID}|sed -n '/gid/,/status/p'|grep status|cut -d'=' -f2)
    if [ "$LinkStatus" = "" ];then
        updateDBValue link "${url}" status "Waiting"
    fi
done

while true
do
sleep 2
Last_Update=$(cat _data/last_updated)
Next_Update=$(expr $Last_Update + 86400)
Current_Date=$(date +%s)
if [ $Current_Date -gt $Next_Update ];then
    _scripts/update_software.sh
    echo "$Next_Update" > _data/last_updated
fi
link=$(selectDBValue links link status "Waiting"|head -1)
if [ "$link" != "" ];then
    Active_Downloads=$(selectDBNumLinksActive)
    if [ $Active_Downloads -eq 0 ];then
        _scripts/download.sh "${link}" &
        sleep 2
    fi
    CurrentMaxJobs=""
    while [ "${CurrentMaxJobs}" = "" ]
    do
        CurrentMaxJobs=$(${XMLRPC} ${SERVER_URL} aria2.getGlobalOption|grep 'max-concurrent-downloads'|cut -d'=' -f2)
    done
    Active_Downloads=$(selectDBNumLinksActive)
    if [ $Active_Downloads -lt $CurrentMaxJobs ];then
        URL=$(selectDBLinksDownloading)
        if [ "$URL" != "" ];then
            hosters=$(echo "$URL"|cut -d'/' -f3|sort|uniq|sed 's|www.||g')
            num_hosters=$(echo "$hosters"|wc -l)
            query=""
            count=1
            for hoster in $(echo $hosters)
            do
                type=$(selectDBValue sitemgr type site "http://${hoster}")
                if [ "$type" != "" ];then
                    count=$(expr $count + 1)
                    if [ $type -eq 0 -o $type -eq 1 ];then
                        query="${query} link NOT LIKE \"%${hoster}%\" ";
                        if [ $num_hosters -gt 1 -a $count -le $num_hosters ];then
                            query="${query} AND"
                        fi
                    fi
                fi
            done
            if [ "${query}" != "" ];then
                query="${query} AND status=\"Waiting\""
            else
                query="status=\"Waiting\""
            fi
            LINK=$(selectDBLinksQuery "$query"|head -1)
            if [ "$LINK" != "" ];then
                _scripts/download.sh "${LINK}" &
            fi
        fi
    fi
fi
sleep 2
done
