#!/bin/sh
# This file is part of Download Manager.
#
# Download Manager is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Download Manager is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Download Manager.  If not, see <http://www.gnu.org/licenses/>.

# Load the common functions
. _scripts/lib.sh

#Change the directory to $homedir
cd $homedir

#Get parameters from config.php
MaxTries=$(selectDBValue preferences value variable no_retries) 
FileAllocation=$(selectDBValue preferences value variable fileallocation) 
AutoClean=$(selectDBValue preferences value variable autoclean) 
ExtractCompleted=$(selectDBValue preferences value variable extractcompleted) 
OverwriteMode=$(selectDBValue preferences value variable overwrite_mode) 
TimeStamp=$(selectDBValue preferences value variable timestamp) 

if [ "$FileAllocation" = "true" ];then
    file_allocation='prealloc'
else
    file_allocation='none'
fi

OutputFileSize=0
downloaddir=""

# Global variables
finallink=""
options=""
FileName=""
link_options=""

selectDBLinksActive()
{
    DB_Locked=1
    while [ $DB_Locked -ne 0 ];do
        Output=$(${SQLITE} ${DBFNAME} "SELECT count(*) FROM links WHERE status=\"Downloading\"" 2>&1)
        DB_Locked=$(echo $Output|grep "database is locked"|wc -l)
    done
    echo "$Output"
}

findErrorMessage()
{
    ErrorCode=$1
    updateDBValue link "${url}" status "Failed"
    if [ $ErrorCode -eq 1 ];then
        updateDBValue link "${url}" file_exists "Error: Unknown"
    elif [ $ErrorCode -eq 2 ];then
        updateDBValue link "${url}" file_exists "Error: Timeout Occured"
    elif [ $ErrorCode -eq 3 ];then
        updateDBValue link "${url}" file_exists "Error: 404 - File Not Found"
    elif [ $ErrorCode -eq 4 ];then
        updateDBValue link "${url}" file_exists "Error: Max File not Found"
    elif [ $ErrorCode -eq 5 ];then
        updateDBValue link "${url}" file_exists "Error: Download Speed is too slow"
    elif [ $ErrorCode -eq 6 ];then
        updateDBValue link "${url}" file_exists "Error: Network Problem"
    elif [ $ErrorCode -eq 7 ];then
        updateDBValue link "${url}" file_exists "Error: Download Aborted"
    elif [ $ErrorCode -eq 8 ];then
        updateDBValue link "${url}" file_exists "Error: Resume not supported"
    fi
}

startDownload()
{
    url="$1"

    NumberOfParts=$(selectDBValue links sections link "$url")

    if [ $NumberOfParts -eq 0 ];then
        NumberOfParts=1
        updateDBValue link "${url}" sections ${NumberOfParts}
    fi

    Referer=$(selectDBValue links referer link "$url")
    if [ "$Referer" = "" ];then
        Referer=$url
    fi

    SpeedLimit=$(selectDBValue links speed_limit link "$url")
    if [ $SpeedLimit -ne 0 ];then
        SpeedLimit=$(echo ${SpeedLimit}K)
    fi

    Input_Param="struct/{file-allocation#$file_allocation#allow-overwrite#$OverwriteMode"
    Input_Param="${Input_Param}#max-tries#$MaxTries#continue#true#remote-time#$TimeStamp"
    Input_Param="${Input_Param}#dir#$downloaddir#split#$NumberOfParts"
    Input_Param="${Input_Param}#max-download-limit#$SpeedLimit"

    if [ "$options" != "" ];then
        Input_Param="${Input_Param}#header#array/$options"
    fi

    Cookies=$(selectDBValue links cookie link "$url")
    if [ "$Cookies" != "" ];then
        Input_Param="${Input_Param}#header#$Cookies"
    fi

    UserAgent=$(selectDBValue links useragent link "$url")
    if [ "$USER_AGENT" != "" ];then
        Input_Param="${Input_Param}#user-agent#$UserAgent"
    fi

    ExtraHeaders=$(selectDBValue links extraheaders link "$url")
    if [ "$ExtraHeaders" != "" ];then
        Input_Param="${Input_Param}#header#$ExtraHeaders"
    fi

    Input_Param="${Input_Param}}"
    Aria2c_StdOut=$(${XMLRPC} ${SERVER_URL} aria2.addUri "array/$finallink" "${Input_Param}")

    GID=$(echo $Aria2c_StdOut|cut -d'"' -f2)
    if [ "$GID" != "" ];then
        updateDBValue link "${url}" gid "$GID"

        sleep 5

        timeout=0
        while [ $OutputFileSize -eq 0 ]
        do
            XMLRPC_Data=$(${XMLRPC} ${SERVER_URL} aria2.tellStatus ${GID})
            OutputFileSizeExists=$(echo "$XMLRPC_Data"|grep 'length='|cut -d'=' -f2|wc -l)
            if [ $OutputFileSizeExists -gt 0 ];then
                OutputFileSize=$(echo "$XMLRPC_Data"|grep 'length='|cut -d'=' -f2)
            fi
            ErrorCode=$(echo "$XMLRPC_Data"|grep errorCode|cut -d'=' -f2)
            if [ "$ErrorCode" = "" ];then
                ErrorCode=0
            else
                break
            fi
            timeout=$(expr $timeout + 2)
            if [ $timeout -eq 70 ];then
                ErrorCode=0
                break
            fi
            sleep 2
        done

        if [ $ErrorCode -eq 0 ];then
            start_date=$(date +%s)
            old_start_date=$(selectDBValue links start_date link "$url")
            curr_date=$start_date
            updateDBValue link "${url}" cstart_date "$curr_date"
    
            if [ $old_start_date -eq 0 ];then
                updateDBValue link "${url}" start_date "$start_date"
            fi

            updateDBValue link "${url}" status "Downloading"
            updateDBValue link "${url}" file_exists "Downloading"

            FileName=$(echo "$XMLRPC_Data"|grep 'path='|cut -d'=' -f2)
            FileName=$(basename "${FileName}")
            updateDBValue link "${url}" filename "$FileName"

            completed=$(FileSize "${downloaddir}${FileName}")
            updateDBValue link "${url}" completed ${completed}

            updateDBValue link "${url}" filesize ${OutputFileSize}

            Connections=$(echo "$XMLRPC_Data"|grep 'connections='|cut -d'=' -f2)
            if [ "$Connections" = "" ];then
                Connections=1
            fi
            updateDBValue link "${url}" sections ${Connections}

            Status="active"
            while [ "${Status}" = "active" ]
            do
                sleep 2
                XMLRPC_Data=$(${XMLRPC} ${SERVER_URL} aria2.tellStatus ${GID})
                Status=$(echo "$XMLRPC_Data"|sed -n '/gid/,/status/p'|grep status|cut -d'=' -f2)
                No_Connections=$(echo "$XMLRPC_Data"|grep 'connections='|cut -d'=' -f2)
                if [ "$No_Connections" != "$Connections" -a "$No_Connections" != "" -a "$No_Connections" != "0" ];then
                    updateDBValue link "${url}" sections ${No_Connections}
                fi
            done
            end_date=$(date +%s)
            updateDBValue link "${url}" end_date "$end_date"

            total_time=$(selectDBValue links total_time link "${url}")
            sess_end_time=$(date +%s)
            sess_time=$(expr $sess_end_time - $curr_date)
            total_time=$(expr $total_time + $sess_time)
        else
            findErrorMessage $ErrorCode
            rm -f ${TMP_COMMAND}
            exit
        fi

        completed=$(FileSize "${downloaddir}${FileName}")
        updateDBValue link "${url}" completed ${completed}

        if [ "${Status}" = "complete" ];then
            if [ "$ExtractCompleted" = "true" ]; then
               password=$(selectDBValue links password link "$url")
               if [ "$password" = "" ];then
                   password="-"
               fi
               _scripts/extract_completed.sh "${downloaddir}${FileName}" "$password" &
            fi
            if [ "$AutoClean" = "true" ];then
                deleteDBRow "${url}"
            else
                updateDBValue link "${url}" status "Complete"
                updateDBValue link "${url}" file_exists "Download Completed"
                updateDBValue link "${url}" total_time $total_time
            fi
        elif [ "${Status}" = "error" ];then
            ErrorCode=$(${XMLRPC} ${SERVER_URL} aria2.tellStatus $GID|grep errorCode|cut -d'=' -f2)
            findErrorMessage $ErrorCode
            updateDBValue link "${url}" total_time $total_time
        else
            updateDBValue link "${url}" status "Pause"
            updateDBValue link "${url}" file_exists "Download Paused"
            updateDBValue link "${url}" total_time $total_time
        fi
    else
        updateDBValue link "${url}" status "Failed"
        updateDBValue link "${url}" file_exists "Download Failed - $Aria2c_StdOut"
    fi
    rm -f ${TMP_COMMAND}
}

#Main Program
options=""
URL="$1"
host=$(echo "$URL"|cut -d'/' -f3|sed 's|www.||g')
sitemgr_data=$(selectDBValue sitemgr "*" site "http://${host}")
state=$(echo "$sitemgr_data"|cut -d'|' -f2)
type=$(echo "$sitemgr_data"|cut -d'|' -f3)
user=$(echo "$sitemgr_data"|cut -d'|' -f4)
pass=$(echo "$sitemgr_data"|cut -d'|' -f5)
plugin_mode=$(selectDBValue links mode link "$URL")
mode=download
downloaddir=$(selectDBValue links location link "$URL")
status=$(selectDBValue links status link "$URL")
if [ "$status" = "Waiting" ];then
    if [ "$user" = "" -o "$pass" = "" -o "$state" = "0" -o "$plugin_mode" = "0" ];then
        finallink="$URL"
        FileName=$(basename "${URL}")
        updateDBValue link "${URL}" filename "$FileName"
        updateDBValue link "${URL}" status "Connecting"
        updateDBValue link "${URL}" file_exists "Connecting to Server..."
        startDownload "${URL}"
        exit
    else
        for hosts in $(ls -1 _scripts/host)
	do
            HNAME=$(echo $hosts|sed 's|\.sh||g')
            PLUGIN=$(echo "$URL"|grep $HNAME|wc -l)
	    if [ "${PLUGIN}" -eq 1 ];then
                updateDBValue link "${URL}" status "Connecting"
                updateDBValue link "${URL}" file_exists "Connecting to Server..."
                is_error=""
                while [ "x${is_error}" != "xError" -a "${status}" != "Pause" ];do
                    link_options=$(_scripts/host/${hosts} "${URL}" $mode $type "$user" "$pass")
		    is_error=$(echo "$link_options"|cut -d':' -f1)
		    if [ "${is_error}" = "http" ];then
		        finallink=$(echo "$link_options"|cut -d'|' -f1)
		        options=$(echo "$link_options"|cut -d'|' -f2)
		        startDownload "${URL}"
                        exit
		    elif [ "${is_error}" = "Reboot" ];then
                        WAIT=$(echo "$link_options"|cut -d':' -f2)
                        updateDBValue link "${URL}" status "Loading"
                        updateDBValue link "${URL}" file_exists "Waiting for other downloads to finish..."
                        NoActiveDownloads=1
                        WaitTimer=0
                        while [ $NoActiveDownloads -ne 0 -a $WaitTimer -lt $WAIT ];do
                            NoActiveDownloads=$(selectDBLinksActive)
                            sleep 2
                            WaitTimer=$(expr $WaitTimer + 2)
                        done
                        if [ $WaitTimer -lt $WAIT ];then
                            updateDBValue link "${URL}" file_exists "Rebooting Router..."
                            _scripts/reboot.sh
                            sleep 50
                        fi
		    elif [ "${is_error}" = "Wait" ];then
                        WAIT=$(echo "$link_options"|cut -d':' -f2)
                        updateDBValue link "${URL}" status "Loading"
                        updateDBValue link "${URL}" file_exists "Loading download in $WAIT seconds..."
                        if [ "${WAIT}" != "" ];then
                            sleep ${WAIT}
                        fi
                    fi
                    status=$(selectDBValue links status link "$URL")
                done
                if [ "${is_error}" = "Error" ];then
                    updateDBValue link "${URL}" status "Failed"
                    updateDBValue link "${URL}" file_exists "${link_options}"
                fi
                exit 1
	    fi
        done
    fi
fi
