#!/bin/sh
# This file is part of Download Manager.
#
# Download Manager is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Download Manager is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Download Manager.  If not, see <http://www.gnu.org/licenses/>.

# Load the common functions
. _scripts/lib.sh

keepbroken=$(selectDBValue preferences value variable keepbroken)
deletearchive=$(selectDBValue preferences value variable deletearchive)
archivedir=$(selectDBValue preferences value variable archivedir)

if [ "$keepbroken" = "true" ];then
    kb="-kb"
fi

#Change the directory to $homedir
cd $homedir

extract_rar() {
    unrarx="${UNRAR} x -o+ -c- ${kb} ${ap} -p${password} -inul "
    Output=`${UNRAR} lt -v -p"${password}" "${filename}" 2>&1`
    errorcode=$?
    isvolume=`echo "$Output"|grep '^Volume' |wc -l`
    if [ $isvolume -gt 0 ]; then
        headercorrupt=`echo "$Output"|grep 'header is corrupt'|wc -l`
        filemissing=`echo "$Output"|grep 'Cannot find volume'|wc -l`
        if [ $headercorrupt -ne 0 -o $errorcode -gt 0 -o $filemissing -gt 0 ]; then
            echo "${filename}: Error while extracting archive. Missing volumes and/or header corrupt."
            exit 3
        fi
    fi
    if [ "$archivedir" != "" ];then
        if [ -d "$archivedir" -a -w "$archivedir" ];then
            cd "$archivedir" && $unrarx "$filename"
        else
            cd "$location" && $unrarx "$filename"
        fi
    fi
    if [ $? -eq 0 ]; then
        # Extraction success
        echo "${filename}: Extraction successful."
        if [ "$deletearchive" = "true" ];then
            OLDIFS=$IFS
            Files=`echo "$Output"|grep '^Volume'|sed 's|^Volume ||g'`
            IFS='
'
            for file in `echo "$Files"`
            do
                rm -f "$file"
                if [ $? -eq 0 ];then
                    echo "${file}: File deleted."
                else
                    echo "${file}: Error deleting file."
                fi
            done
            IFS=$OLDIFS
        fi
    else
        echo "${filename}: Extraction failed."
        cd $homedir
    fi
    exit
}

filename="$1"
# Find the first part of the archive
input_partno=`echo "$filename"|grep 'part'|sed 's|.rar.*||g'|sed 's|.*part||g'`
if [ "$input_partno" != "" ];then
    num_digits=`printf "$input_partno"|wc -c|sed 's| ||g'`
    first_part=`printf "%0${num_digits}d\n" 1`
    filename=`echo "$filename"|sed "s|part${input_partno}|part${first_part}|g"`
fi
password="$2"
location=`dirname "$filename"`
case "$filename" in
    *.rar|*.rar.*)
        if [ -d "$location" ]; then 
            cd "$location"
        else 
            echo "${filename}: File not accessible"
            exit 1
        fi
        extract_rar
    ;;
    *)    exit 2;
    ;;
esac
