#!/bin/sh
# This file is part of Download Manager.
#
# Download Manager is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Download Manager is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Download Manager.  If not, see <http://www.gnu.org/licenses/>.

URL=$1
MODE=$2
TYPE=$3
USER=$4
PASS=$5

# Load the common functions
. _scripts/lib.sh

# $1: Html data
# $2: URL
# $3: Cookies File
# stdout: Html Data
solve_recaptcha() {
    html_data="$1"
    URL="$2"
    COOKIE="$3"
    PUBKEY='6LdSvrkSAAAAAOIwNj-IY-Q-p90hQrLinRIpZBPi'
    SHORT=$(basename_file "$URL")
    IMAGE_FILENAME=$(recaptcha_load_image $PUBKEY)
    CHALLENGE=$(recaptcha_get_challenge_from_image "$IMAGE_FILENAME")
    if [ -f "$IMAGE_FILENAME" ];then
        WORD_WITH_TICKET=$(submit_captcha_trader "$IMAGE_FILENAME")
        TICKET_ID=$(echo "$WORD_WITH_TICKET"|cut -d':' -f1)
        WORD=$(echo "$WORD_WITH_TICKET"|cut -d':' -f2)
        if [ "$TICKET_ID" = "-1" ];then
            log_debug "$URL" "Captcha Trader Error"
            echo "Error: Captcha Trader Error"
            rm -f $IMAGE_FILENAME
            exit 1
        else
            log_debug "$URL" "Captcha Trader Output: $WORD"
            html_data=$($WGET -qO- --header="$COOKIE" --referer="$URL" --post-data="recaptcha_challenge_field=$CHALLENGE&recaptcha_response_field=$WORD&recaptcha_shortencode_field=$SHORT" "http://www.fileserve.com/checkReCaptcha.php")
            respond_captcha_trader true $TICKET_ID
            rm -f $IMAGE_FILENAME
        fi
    fi
    echo "$html_data"
}

if [ "$MODE" = "download" ];then
    #
    # Download Mode
    #
    if [ $TYPE -eq 0 ];then
        ID=$(echo "$URL" | parse_quiet '\/file\/' 'file\/\([^/]*\)')
        if [ "$ID" = "" ];then
            echo "Error: Cannot parse URL to extract file id (mandatory)"
            log_debug "$URL" "Cannot parse URL to extract file id (mandatory)"
            exit 1
        fi

        NEWURL="http://www.fileserve.com/file/$ID"

        updateDBValue link "${URL}" status "Loading"

        MAINPAGE=$($WGET -S -O- "$NEWURL" 2>&1)
        if [ "$MAINPAGE" = "" ];then
            echo "Error: Cannot get Download Page"
            log_debug "$URL" "Cannot get Download Page"
            exit 1
        fi
        COOKIE=$(echo "$MAINPAGE"|grep 'Set-Cookie:'|sed 's|  Set-Cookie: ||g'|sed 's|path=/|path=/;|g'|tr '\n' ' ')

        # "The file could not be found. Please check the download link."
        if match 'File not available' "$MAINPAGE"; then
            echo "Error: File not found"
            log_debug "$URL" "File not found."
            exit 1
        fi

        JSON1=$($WGET -qO- --header="Cookie: $COOKIE" --referer="$NEWURL" --post-data="checkDownload=check" "$NEWURL")

        if match 'waitTime' "$JSON1"; then
            check_router 0 "File temporarily unavailable"
            log_debug "$URL" "File temporarily unavailable"
            exit 1
        elif match 'timeLimit' "$JSON1"; then
            check_router 0 "Time Limit, You must wait"
            log_debug "$URL" "Time Limit, You must wait"
            exit 1
        elif match 'parallelDownload' "$JSON1"; then
            echo "Error: Your IP address is already downloading a file."
            exit 1
        elif ! match 'success' "$JSON1"; then
            echo "Error: Unexpected error, site update?"
            log_debug "$URL" "Unexpected error, site update?"
            exit 1
        fi

        updateDBValue link "${URL}" file_exists "Checking credits in captchatrader.com."
        log_debug "$URL" "Checking credits in captchatrader.com."
        credits=$(getcredits_captcha_trader)
        if [ $credits -ge 10 ];then
            log_debug "$URL" "Found $credits credits in captchatrader.com."
            updateDBValue link "${URL}" file_exists "Solving reCaptcha using captchatrader.com"
            PAGE=$(solve_recaptcha "$PAGE" "$NEWURL" "Cookie: $COOKIE")
            if [ "$PAGE" = "" ];then
                echo "Error: Captcha Error"
                log_debug "$URL" "Error in solving reCaptcha"
                exit 1
            fi
            if match 'Please Enter Captcha' "$PAGE"; then
                echo "Error: Captcha Error"
                exit 1
            fi
            if match 'Error: Captcha Trader Error' "$PAGE"; then
                echo "Error: Captcha Trader Error"
                exit 1
            fi

            RES=$(echo "$PAGE" | parse_quiet 'success' 'success"\?[[:space:]]\?:[[:space:]]\?\([[:digit:]]*\)')
            if [ "$RES" != "1" ] ; then
                echo "Error: Wrong Captcha, Try again."
                log_debug "$URL" "Wrong Captcha, Try Again."
                exit 1
            fi
        else
            echo "Error: You don't have enough credits in captchatrader.com"
            log_debug "$URL" "You don't have enough credits in captchatrader.com."
            exit 1
        fi

        MSG1=$($WGET -qO- --header="Cookie: $COOKIE" --referer="$NEWURL" --post-data="downloadLink=wait" "$NEWURL")
        if match 'fail404' "$MSG1"; then
            echo "Error: Unexpected Result"
            log_debug "$URL" "Unexpected Result"
            exit 1
        fi

        WTIME=$(echo "$MSG1" | cut -c4-)
        if [ "$WTIME" = "" ];then
            echo "Error: Unable to get wait time."
            log_debug "$URL" "Unable to get wait time."
            exit 1
        fi
       
        log_debug "$URL" "Loading download in $WTIME seconds..."
        updateDBValue link "${URL}" file_exists "Loading download in $WTIME seconds..."
        sleep ${WTIME}

        MSG2=$($WGET -qO- --header="Cookie: $COOKIE" --referer="$NEWURL" --post-data="downloadLink=show" "$NEWURL")
        if [ "$MSG2" = "" ];then
            echo "Error: URL error in fileserve.com"
            log_debug "$URL" "URL Error in fileserve.com."
            exit 1
        fi

        FILE_URL=$($WGET --spider -S --header="Cookie: $COOKIE" --referer="$NEWURL" --post-data="download=normal" "$NEWURL" 2>&1| grep_http_header_location)
        if [ "$FILE_URL" = "" ];then
            echo "Error: URL error in fileserve.com"
            log_debug "$URL" "URL Error in fileserve.com."
            exit 1
        fi

        echo "${FILE_URL}|"
        exit 1
    elif [ $TYPE -eq 1 ];then
        #
        # Free Account
        #
        echo "Error: Free Not yet implemented||"
        exit 1
    else
        #
        # Premium Account
        #
        echo "Error: Premium Not yet implemented||"
        exit 1
    fi
elif [ "$MODE" = "verify" ];then
    echo "Error: Not yet Implemented. Please PM your credentials to the author to write the premium plugin."
    exit 1
fi
