#!/bin/sh
# set -x
# This file is part of Download Manager.
#
# Download Manager is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Download Manager is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Download Manager.  If not, see <http://www.gnu.org/licenses/>.

URL=$1
MODE=$2
TYPE=$3
USER=$4
PASS=$5

# Load the common functions
. _scripts/lib.sh

# $1: Html data
# $2: URL
# $3: COOKIES File
# stdout: HTML data
solve_recaptcha() {
    html_data="$1"
    URL="$2"
    COOKIE="$3"
    PUBKEY='6LdNWbsSAAAAAIMksu-X7f5VgYy8bZiiJzlP83Rl'
    IMAGE_URL=$(recaptcha_load_image $PUBKEY)
    CHALLENGE=$(recaptcha_get_challenge_from_image "$IMAGE_URL")
    if [ "$IMAGE_URL" != "" ];then
        WORD_WITH_TICKET=$(submit_captcha_trader "$IMAGE_URL")
        TICKET_ID=$(echo "$WORD_WITH_TICKET"|cut -d':' -f1)
        if [ "$TICKET_ID" = "" ];then
            log_debug "$URL" "Empty response from Captcha Trader"
            echo "Error: Empty response from Captcha Trader"
            exit 1
        fi
        WORD=$(echo "$WORD_WITH_TICKET"|cut -d':' -f2)
        if [ "$TICKET_ID" = "-1" ];then
            log_debug "$URL" "Captcha Trader Error"
            echo "Error: Captcha Trader Error"
            exit 1
        else
            log_debug "$URL" "Captcha Trader Output: $WORD"
            html_data=$(wget --user-agent="$USER_AGENT" -qO- --header="$COOKIE" --header="X-Requested-With: XMLHttpRequest" --referer="$URL" --post-data="recaptcha_challenge_field=$CHALLENGE&recaptcha_response_field=$WORD" "$URL?start=1") || return 1
            if [ "$html_data" = "" ];then
                respond_captcha_trader false $TICKET_ID
            else
                respond_captcha_trader true $TICKET_ID
            fi
        fi
    fi
    echo "$html_data"
}

if [ "$MODE" = "download" ];then
    #
    # Download Mode
    #
    if [ $TYPE -eq 0 ];then
        ID=$(echo "$URL" | parse_quiet '\/file\/' 'file\/\([^/]*\)')
        if [ "$ID" = "" ];then
            echo "Error: Cannot parse URL to extract file id (mandatory)"
            log_debug "$URL" "Cannot parse URL to extract file id (mandatory)"
            exit 1
        fi
        BASEURL=$(basename_url "$(wget --user-agent="$USER_AGENT" --spider -S "$(basename_url "$URL")" 2>&1|grep_http_header_location)")
        NEWURL="$BASEURL/file/$ID"

        MAINPAGE=$(wget -S -O- "$NEWURL" 2>&1) 
        if [ "$MAINPAGE" = "" ];then
            echo "Error: Cannot get Main Page"
            log_debug "$URL" "Cannot get Main Page"
            exit 1
        fi
        COOKIE=$(echo "$MAINPAGE"|grep 'Set-Cookie:'|sed 's|  Set-Cookie: |Cookie: |g')
    
        PAGE=$(wget --user-agent="$USER_AGENT" -qO- --header="$COOKIE" --header="X-Requested-With: XMLHttpRequest" --referer="$NEWURL?start=1" --post-data="" "$NEWURL?start=1") 
        if [ "$PAGE" = "" ];then
            echo "Error: Cannot get Download Page"
            log_debug "$URL" "Cannot get Download Page"
            exit 1
        fi
    
        if match 'File does not exist' "$PAGE"; then
            echo "Error: File not found"
            log_debug "$URL" "File not found"
            exit 1
        fi

        FILE_URL=""
        FOLLOWS=0
        #
        # Try for 5 times and quit
        #
        updateDBValue link "${URL}" status "Loading"
        while [ "$FILE_URL" = "" -a "$FOLLOWS" -lt 5 ]; do
            FOLLOWS=$(expr $FOLLOWS + 1)
            log_debug "$URL" "Attempt $FOLLOWS"
            if match 'Start download now' "$PAGE"; then
                # download link
                log_debug "$URL" "Found Download Link."
                FILE_URL=$(echo $PAGE |parse_quiet 'Start download now' 'href="\([^"]*\)"')
            elif match 'Please Enter Captcha' "$PAGE"; then
                # captcha
                log_debug "$URL" "Found reCaptcha."
                if [ "$autorecaptcha" = "true" ];then
                    updateDBValue link "${URL}" file_exists "Checking credits in captchatrader.com."
                    credits=$(getcredits_captcha_trader)
                    log_debug "Checking credits in captchatrader.com."
                    if [ $credits -ge 10 ];then
                        log_debug "Found $credits in captchatrader.com."
                        updateDBValue link "${URL}" file_exists "Solving reCaptcha using captchatrader.com"
                        PAGE=$(solve_recaptcha "$PAGE" "$NEWURL" "$COOKIE")
                        if [ "$PAGE" = "" ];then
                            echo "Error: Captcha Error"
                            log_debug "$URL" "Error in solving reCaptcha"
                            exit 1
                        fi
                        if match 'Please Enter Captcha' "$PAGE"; then
                            echo "Error: Wrong Captcha"
                            exit 1
                        elif match 'Error: Captcha Trader Error' "$PAGE"; then
                            echo "Error: Captcha Trader Error"
                            exit 1
                        fi
                    else
                        echo "Error: You don't have enough credits in captchatrader.com"
                        log_debug "You don't have enough credits in captchatrader.com."
                        exit 1
                    fi
                else
                    echo "Error: Auto-reCAPTCHA not enabled."
                    log_debug "Auto-reCAPTCHA not enabled."
                    exit 1
                fi
            elif match 'countDownDelay' "$PAGE"; then
                # wait
                WTIME=$(echo "$PAGE" |parse_quiet 'var countDownDelay = ' 'countDownDelay = \([0-9]*\);') 
                if [ "$WTIME" = "" ];then
                    echo "Error: Unable to get wait time."
                    log_debug "$URL" "Unable to get wait time."
                    exit 1
                fi
        
                if [ $WTIME -gt 40 ];then
                    check_router "$WTIME" "You need to wait for $WTIME seconds"
                    exit 1
                else
                    log_debug "$URL" "Loading download in $WTIME seconds"
                    updateDBValue link "${URL}" file_exists "Loading download in $WTIME seconds..."
                    sleep ${WTIME}
                fi
    
                # for wait time > 5min. these values may not be present
                # it just means we need to try again so the following code is fine
                TM=$(echo "$PAGE" |parse_quiet "name='tm' value='" "name='tm' value='\([0-9]*\)'")
                TM_HASH=$(echo "$PAGE" |parse_quiet "name='tm_hash' value='" "name='tm_hash' value='\([a-f0-9]*\)'")

                PAGE=$(wget --user-agent="$USER_AGENT" -S -O- --header="$COOKIE" --header="X-Requested-With: XMLHttpRequest" --referer="$NEWURL" --post-data="tm=$TM&tm_hash=$TM_HASH" "$NEWURL?start=1" 2>&1)
                if [ "$PAGE" = "" ];then
                    echo "Error: Unable to get Final URL."
                    log_debug "$URL" "Unable to get Final URL."
                    exit 1
                fi
            else
                echo "Error: No match. Site update?"
                log_debug "$URL" "No match. Site update?"
                exit 1
            fi
        done
        if [ "${FILE_URL}" = "" ];then
            echo "Error: URL error in filesonic.com"
            log_debug "$URL" "URL Error in filesonic.com."
            exit 1
        fi
        echo "${FILE_URL}|${COOKIE}"
        exit 1
    elif [ $TYPE -eq 1 ];then
        #
        # Free Account
        #
        echo "Error: Free Not yet implemented||"
        exit 1
    else
        #
        # Premium Account
        #
        echo "Error: Premium Not yet implemented||"
        exit 1
    fi
elif [ "$MODE" = "verify" ];then
    echo "Error: Not yet Implemented. Please PM your credentials to the author to write the premium plugin."
    exit 1
fi
