#!/bin/sh
# This file is part of Download Manager.
#
# Download Manager is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Download Manager is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Download Manager.  If not, see <http://www.gnu.org/licenses/>.

URL=$1
MODE=$2
TYPE=$3
USER=$4
PASS=$5
BASE_URL='http://hotfile.com'

# Load the common functions
. _scripts/lib.sh

# $1: Html data
# stdout: reCaptcha String
solve_recaptcha() {
    form2_url="$1"
    form2_action="$2"
    COOKIE="$3"
    PUBKEY='6LfRJwkAAAAAAGmA3mAiAcAsRsWvfkBijaZWEvkD'
    IMAGE_URL=$(recaptcha_load_image $PUBKEY)
    CHALLENGE=$(recaptcha_get_challenge_from_image "$IMAGE_URL")
    if [ "$IMAGE_URL" != "" ];then
        WORD_WITH_TICKET=$(submit_captcha_trader "$IMAGE_URL")
        TICKET_ID=$(echo "$WORD_WITH_TICKET"|cut -d':' -f1)
        if [ "$TICKET_ID" = "" ];then
            log_debug "$URL" "Empty response from Captcha Trader"
            echo "Error: Empty response from Captcha Trader"
            exit 1
        fi
        WORD=$(echo "$WORD_WITH_TICKET"|cut -d':' -f2)
        if [ "$TICKET_ID" = "-1" ];then
            log_debug "$URL" "Captcha Trader Error"
            echo "Error: Captcha Trader Error"
            exit 1
        else
            log_debug "$URL" "Captcha Trader Output: $WORD"
            html_data=$(wget --header="$COOKIE" --post-data="action=${form2_action}&recaptcha_challenge_field=${CHALLENGE}&recaptcha_response_field=${WORD}" "${BASE_URL}${form2_url}" -qO-)
            if [ "$html_data" = "" ];then
                respond_captcha_trader false $TICKET_ID
            else
                respond_captcha_trader true $TICKET_ID
            fi
        fi
    fi
    echo "$html_data"
}

if [ "$MODE" = "download" ];then
    #
    # Download Mode
    #
    # Check the URL for existence
    WAIT_HTML=$(wget -S -O- "$URL" 2>&1)

    # "This file is either removed due to copyright claim or is deleted by the uploader."
    if match '\(404 - Not Found\|or is deleted\)' "$WAIT_HTML"; then
        log_debug "$URL" "File not found"
        echo "Error: File not found"
        exit 1
    fi
    if [ "$USER" != "none" -o "$PASS" != "none" ]; then
        FILE_URL=$(wget "http://api.hotfile.com/?action=getdirectdownloadlink&username=${USER}&password=${PASSWORD}&link=${URL}")
        # Hotfile API error messages starts with a dot, if no dot then the download link is available
        if [ ${FILE_URL:0:1} == "." ]; then
            log_debug "login request failed (bad login/password or link invalid/removed)"
            echo "Error: Login request failed (bad login/password or link invalid/removed)"
            exit 1
        fi

        echo "$FILE_URL|"
        exit 1
    fi
    COOKIE=$(echo "$WAIT_HTML"|grep 'Set-Cookie:'|sed 's|  Set-Cookie: |Cookie: |g')

    if match 'You are currently downloading' "$WAIT_HTML"; then
        echo "Error: You are currently downloading. Only one connection with server allow for free users."
        exit 1
    else
        SLEEP=$(echo "$WAIT_HTML" | parse_last 'timerend=d.getTime()' '+\([[:digit:]]\+\);')
        if [ "$SLEEP" = "" ];then
            log_debug "$URL" "can't get sleep time"
            echo "Error: Hotfile - cannot get sleep time"
            exit 1
        fi
        SLEEP=`expr $SLEEP \/ 1000`
        check_router "$SLEEP" "You have reached your hourly traffic limit."

        FORM_HTML=$(grep_form_by_name "$WAIT_HTML" 'f')
        form_url=$(echo "$FORM_HTML" | parse_form_action)
        form_action=$(echo "$FORM_HTML" | parse_form_input_by_name 'action')
        form_tm=$(echo "$FORM_HTML" | parse_form_input_by_name 'tm')
        form_tmhash=$(echo "$FORM_HTML" | parse_form_input_by_name 'tmhash')
        form_wait=$(echo "$FORM_HTML" | parse_form_input_by_name 'wait')
        form_waithash=$(echo "$FORM_HTML" | parse_form_input_by_name 'waithash')
        form_upidhash=$(echo "$FORM_HTML" | parse_form_input_by_name 'upidhash')

        if [ "$form_wait" != "" ];then
            #
            # Update Database Values
            #
            updateDBValue link "${URL}" status "Loading"
            updateDBValue link "${URL}" file_exists "Loading download in $form_wait seconds..."
            #
            # Wait for form_wait Seconds
            #
            sleep ${form_wait}
        fi
        WAIT_HTML2=$(wget --header="$COOKIE" --post-data="action=${form_action}&tm=${form_tm}&tmhash=${form_tmhash}&wait=${form_wait}&waithash=${form_waithash}&upidhash=${form_upidhash}" "$URL" -qO-)

        # Direct download (no captcha)
        if match 'Click here to download' "$WAIT_HTML2"; then
            FILEURL=$(echo "$WAIT_HTML2" | parse_attr 'click_download' 'href')
            echo "${FILEURL}|"
            exit 1
        # reCaptcha page
        elif match 'api\.recaptcha\.net' "$WAIT_HTML2"; then
            FORM2_HTML=$(grep_form_by_order "$WAIT_HTML2" 2)
            form2_url=$(echo "$FORM2_HTML" | parse_form_action)
            form2_action=$(echo "$FORM2_HTML" | parse_form_input_by_name 'action')
            log_debug "$URL" "Found reCaptcha."
            if [ "$autorecaptcha" = "true" ];then
                updateDBValue link "${URL}" file_exists "Checking credits in captchatrader.com."
                credits=$(getcredits_captcha_trader)
                log_debug "Checking credits in captchatrader.com."
                if [ $credits -ge 10 ];then
                    log_debug "Found $credits in captchatrader.com."
                    updateDBValue link "${URL}" file_exists "Solving reCaptcha using captchatrader.com"
                    PAGE=$(solve_recaptcha "$form2_url" "form2_action" "$COOKIE")
                    if [ "$PAGE" = "" ];then
                        echo "Error: Captcha Error"
                        log_debug "$URL" "Error in solving reCaptcha"
                        exit 1
                    fi
                    if match 'Wrong Code. Please try again' "$PAGE"; then
                        echo "Error: Wrong Captcha"
                        exit 1
                    elif match 'Error: Captcha Trader Error' "$PAGE"; then
                        echo "Error: Captcha Trader Error"
                        exit 1
                    fi
                else
                    echo "Error: You don't have enough credits in captchatrader.com"
                    log_debug "You don't have enough credits in captchatrader.com."
                    exit 1
                fi
            else
                echo "Error: Auto-reCAPTCHA not enabled."
                log_debug "Auto-reCAPTCHA not enabled."
                exit 1
            fi
            FILEURL=$(echo "$PAGE" | parse_attr 'click_download' 'href')
            if [ "$FILEURL" != "" ];then
                echo "${FILEURL}|"
            else
                echo "Error: Hotfile script error."
                exit 1
            fi
        fi
    fi
elif [ "$MODE" = "verify" ];then
    echo "Error: Not yet Implemented. Please PM your credentials to the author to write the premium plugin."
    exit 1
fi
