#!/bin/sh
# This file is part of Download Manager.
#
# Download Manager is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Download Manager is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Download Manager.  If not, see <http://www.gnu.org/licenses/>.

URL=$1
MODE=$2
TYPE=$3
USER=$4
PASS=$5

# Load the common functions
. _scripts/lib.sh

if [ "$MODE" = "download" ];then
    #
    # Download Mode
    #
    # URL Format: http://rapidshare.com/#!download|774tl4|429794114|arc02f.rar|5249
    if expr match "$URL" '.*/#!download|' >/dev/null; then
        FILEID=$(echo "$URL" |cut -d'|' -f3)
        FILENAME=$(echo "$URL" |cut -d'|' -f4)
    else
    # URL Format: http://rapidshare.com/files/429795114/arc02f.rar
        FILEID=$(echo "$URL" |cut -d'/' -f5)
        FILENAME=$(echo "$URL" |cut -d'/' -f6)
    fi

    if [ "$FILEID" = "" -o "$FILENAME" = "" ];then
        echo "Error: Cannot parse fileID/filename from URL: $URL"
        log_debug "$URL" "Cannot parse fileID/filename from URL"
        exit 1
    fi

    APIURL="https://api.rapidshare.com/cgi-bin/rsapi.cgi?sub=download_v1&fileid=${FILEID}&filename=${FILENAME}"

    if [ "$USER" = "none" -o "$PASS" = "none" ]; then
        PARAMS=""
    else
        PARAMS="&login=$USER&password=$PASS"
    fi

    PAGE=$(wget -qO- --no-check-certificate "${APIURL}${PARAMS}")
    if [  "$PAGE" = "" ];then
        echo "Error: Cannot get main API page"
        log_debug "$URL" "Cannot get main API page"
        exit 1
    fi
    ERROR=$(echo "$PAGE" | parse_quiet "ERROR:" "ERROR:[[:space:]]*\(.*\)")

    if match "need to wait" "$ERROR"; then
        WAIT=$(echo "$ERROR" | parse "." "wait \([[:digit:]]\+\) seconds")
        if [ "$WAIT" = "" ];then
            echo  "Error: Cannot parse wait time: $ERROR"
            exit 1
        fi
        log_debug "$URL" "$ERROR"
        check_router "$WAIT" "$ERROR"
        exit 1
    elif match "File \(deleted\|not found\|ID invalid\)" "$ERROR"; then
        log_debug "$URL" "$ERROR"
        echo "Error: $ERROR"
        exit 1
    elif match "flooding" "$ERROR"; then
        log_debug "$URL" "$ERROR"
        check_router 0 "$ERROR"
        exit 1
    elif match "slots" "$ERROR"; then
        log_debug "$URL" "Server has said there is no free slots available"
        echo "Error: Server has said there is no free slots available"
        exit 1
    elif match "need RapidPro" "$ERROR"; then
        log_debug "$URL" "$ERROR"
        check_router 0 "$ERROR"
        exit 1
    elif test "$ERROR"; then
        log_debug "$URL" "$ERROR"
        echo "Error: $ERROR"
        exit 1
    fi

    VALUES=$(echo "$PAGE" | grep "^DL:" | cut -d":" -f2- | awk -F"," '{print $1, $2, $3}')
    RSHOST=$(echo $VALUES|cut -d' ' -f1)
    DLAUTH=$(echo $VALUES|cut -d' ' -f2)
    WTIME=$(echo $VALUES|cut -d' ' -f3)
    if [ "$RSHOST" = "" -o "$DLAUTH" = "" -o "$WTIME" = "" ];then
        echo  "Error: Unexpected page contents"
        log_debug "$URL" "Unexpected page contents"
        exit 1
    fi

    log_debug "$URL" "Loading download in $WTIME seconds"
    updateDBValue link "${URL}" status "Loading"
    updateDBValue link "${URL}" file_exists "Loading download in $WTIME seconds..."
    sleep ${WTIME}
    
    BASEURL="http://${RSHOST}/cgi-bin/rsapi.cgi?sub=download_v1"

    if [ "$USER" = "none" -o "$PASS" = "none" ]; then
        echo "${BASEURL}&fileid=${FILEID}&filename=${FILENAME}&dlauth=${DLAUTH}|"
    else
        echo "${BASEURL}&fileid=${FILEID}&filename=${FILENAME}&login=${USER}&password=${PASS}|"
    fi
elif [ "$MODE" = "verify" ];then
    # Set post-data
    post_data="sub=getaccountdetails_v1&withcookie=1&type=prem&login=${USER}&password=${PASS}"

    # Fetch login data
    html_data=$(wget https://api.rapidshare.com/cgi-bin/rsapi.cgi --no-check-certificate --save-headers --post-data=${post_data} -qO-)
    Output=$(echo "$html_data"|grep 'Login failed')
    if [ "$Output" != "" ];then
        echo "Error: Login failed. Password incorrect."
        exit 1
    fi
    rs_cookie=$(echo "$html_data"|grep 'cookie='|cut -d'=' -f2)

    if [ "$rs_cookie" != "user=" ];then
        echo "$rs_cookie" > "${TMPDIR}/.rs_cookie_user_value"
        echo "Verified."
    else
        echo "Error: Failed to get cookie."
        exit 1
    fi
fi
