#!/bin/sh
# This file is part of Download Manager.
#
# Download Manager is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Download Manager is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Download Manager.  If not, see <http://www.gnu.org/licenses/>.

homedir=$(cat _include/config.php|grep '"homedir"] ='|cut -d'"' -f4)
utildir=$(cat _include/config.php|grep '"utildir"] ='|cut -d'"' -f4)

TMPDIR=${homedir}_tmp
DBFNAME="_data/popdown.db"
TMP1=$TMPDIR/tmp.$$
TMP2=$TMPDIR/tmp2.$$
TMP_IMG=$TMPDIR/tmpimg.$$
TMP_COMMAND=$TMPDIR/command.run.$$

#
# Mandotary Packages
#
A2CNFO=${utildir}/bin/aria2nfo
ARIA2C=${utildir}/bin/aria2c
XMLRPC=${utildir}/bin/aria2-xmlrpc
SERVER_URL="http://localhost:6800/rpc"
SQLITE=${utildir}/bin/sqlite3
CURL=${utildir}/bin/curl

#
# Optional Packages
#
UNRAR=${utildir}/bin/unrar
CONVERT=${utildir}/bin/convert
TESSERACT=${utildir}/bin/tesseract
PERL=${utildir}/bin/perl

# Return the Select Field Value
# $1: Select FIELD NAME
# $2: Set FIELD NAME
# $3: Set FIELD VALUE
# stdout: Select FIELD VALUE
selectDBValue() {
    table=$1
    select=$2
    option=$3
    value=$4
    DB_Locked=1
    while [ $DB_Locked -ne 0 ];do
        Output=$(${SQLITE} ${DBFNAME} "SELECT ${select} FROM ${table} WHERE ${option}=\"${value}\"" 2>&1)
        DB_Locked=$(echo $Output|grep "database is locked"|wc -l)
    done
    echo "$Output"
}

#utildir=$(selectDBValue preferences value variable utildir)
routertype=$(selectDBValue preferences value variable routertype)
rebootrouter=$(selectDBValue preferences value variable rebootrouter)
api_key=$(selectDBValue preferences value variable api_key)
captchauser=$(selectDBValue preferences value variable captchauser)
captchapass=$(selectDBValue preferences value variable captchapass)
autocaptcha=$(selectDBValue preferences value variable autocaptcha)
autorecaptcha=$(selectDBValue preferences value variable autorecaptcha)
RECAPTCHA_SERVER="http://www.google.com/recaptcha/api/"
CT_SUBMIT_API_URL="http://captchatrader.com/api/submit"
CT_RESPOND_API_URL="http://captchatrader.com/api/respond"
#XMLRPC_TRACE_XML=1;export XMLRPC_TRACE_XML
USER_AGENT="Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.10) Gecko/2009042523 Ubuntu/9.04 (jaunty) Firefox/3.0.10"

PATH=/usr/bin:/bin:/usr/sbin:/sbin:/opt/syb/sigma/bdj/jvm/bin:/bin:/sbin:/usr/bin:/usr/sbin:/usr/bin/X11:/usr/local/bin:/opt/syb/app/bin:/opt/syb/app/sbin:/opt/syb/sigma/bin:/usr/local/bin:/share/Apps/utils/bin/:/opt/syb/sigma/bdj/jvm/bin:/bin:/sbin:/usr/bin:/usr/sbin:/usr/bin/X11:/usr/local/bin:/opt/syb/app/bin:/opt/syb/app/sbin:/opt/syb/sigma/bin:/usr/local/bin:/share/Apps/lighttpd/bin:${utildir}bin/;export PATH
LD_LIBRARY_PATH=$LD_LIBRARY_PATH:/lib:/opt/syb/sigma/bdj:/opt/syb/sigma/bdj/jvm/lib:/opt/syb/sigma/bdj/bdj/lib:/opt/syb/app/lib:/opt/syb/sigma/lib:/usr/local/lib:/nmt/apps/lib:${utildir}lib/;export LD_LIBRARY_PATH

# Substring replacement (replace all matches)
#
# stdin: input string
# $1: substring to find (this is not a regexp)
# $2: replacement string (this is not a regexp)
replace() {
    S="$(cat)"
    # We must escape '\' character
    FROM="${1//\\/\\\\}"
    echo "${S//$FROM/$2}"
}

# Delete leading and trailing spaces, tabs, \r, ...
# stdin: input string
# stdout: result string
strip() {
    cat | sed 's/^[[:space:]]*//; s/[[:space:]]*$//'
}

# Return uppercase string
uppercase() {
    tr '[a-z]' '[A-Z]'
}

# Check existance of executable in path
# Better than "which" (external) executable
#
# $1: Executable to check
# $?: zero means not found
check_exec() {
    type -P $1 >/dev/null || return 1 && return 0
}

# Check if a string ($2) matches a regexp ($1)
# This is case sensitive.
#
# $? is zero on success
match() {
    #grep -q "$1" <<< "$2"
    echo "$2" | grep -q "$1"
}

# Check if a string ($2) matches a regexp ($1)
# This is not case sensitive.
#
# $? is zero on success
matchi() {
    #grep -iq "$1" <<< "$2"
    echo "$2" | grep -iq "$1"
}

# Get first line that matches a regular expression and extract string from it.
#
# stdin: text data
# $1: POSIX-regexp to filter (get only the first matching line).
# $2: POSIX-regexp to match (use parenthesis) on the matched line.
parse_all() {
    STRING=$(sed -n "/$1/s/^.*$2.*$/\1/p") &&
        test "$STRING" && echo "$STRING"
}

# Like parse_all, but get only first match
parse() {
    parse_all "$@" | head -n1
}

# Like parse_all, but get only last match
parse_last() {
    parse_all "$@" | tail -n1
}

# Like parse, but hide output to stderr
parse_quiet() {
    parse "$@" 2>/dev/null
}

# Grep first "Location" (of http header)
#
# stdin: result of curl request (with -i/--include, -D/--dump-header or
#        or -I/--head flag)
grep_http_header_location() {
    sed -n 's/^[Ll]ocation:[[:space:]]\+\([^ ]*\)/\1/p' 2>/dev/null | tr -d "\r"
}

# Grep first "Content-Disposition" (of http header)
#
# stdin: same as grep_http_header_location() below
grep_http_header_content_disposition() {
    parse "[Cc]ontent-[Dd]isposition:" 'filename="\(.*\)"' 2>/dev/null
}

# Extract a specific form from a HTML content.
# We assume here that start marker <form> and end marker </form> are one separate lines.
# HTML comments are just ignored. But it's enough for our needs.
#
# $1: (X)HTML data
# $2: (optionnal) Nth <form> (default is 1)
# stdout: result
grep_form_by_order() {
    local DATA="$1"
    local N=${2:-"1"}

     while [ "$N" -gt "1" ]; do
         ((N--))
         DATA=$(echo "$DATA" | sed -ne '/<\/form>/,$p' | sed -e '1s/<\/form>/<_form>/1')
     done

     # FIXME: sed will be greedy, if other forms are remaining they will be returned
     echo "$DATA" | sed -ne '/<form /,/<\/form>/p'
}

# Extract a named form from a HTML content.
# If several forms have the same name, take first one.
#
# $1: (X)HTML data
# $2: "name" attribute of <form> marker
# stdout: result
grep_form_by_name() {
    local DATA="$1"

    if [ -n "$2" ]; then
        # FIXME: sed will be greedy, if other forms are remaining they will be returned
        echo "$DATA" | sed -ne "/<[Ff][Oo][Rr][Mm][[:space:]].*name=\"\?$2\"\?/,/<\/[Ff][Oo][Rr][Mm]>/p"
    fi
}

# Extract a id-specified form from a HTML content.
# If several forms have the same id, take first one.
#
# $1: (X)HTML data
# $2: "id" attribute of <form> marker
# stdout: result
grep_form_by_id() {
    local DATA="$1"

    if [ -n "$2" ]; then
        # FIXME: sed will be greedy, if other forms are remaining they will be returned
        echo "$DATA" | sed -ne "/<[Ff][Oo][Rr][Mm][[:space:]].*id=\"\?$2\"\?/,/<\/[Ff][Oo][Rr][Mm]>/p"
    fi
}

# Split into several lines html markers.
# Insert a new line after ending marker.
#
# stdin: (X)HTML data
# stdout: result
break_html_lines() {
    sed 's/\(<\/[^>]*>\)/\1\n/g'
}

# Split into several lines html markers.
# Insert a new line after each (beginning or ending) marker.
#
# stdin: (X)HTML data
# stdout: result
break_html_lines_alt() {
    sed 's/\(<[^>]*>\)/\1\n/g'
}

# Return value of html attribute
parse_attr() {
    parse "$1" "$2=[\"']\?\([^\"'>]*\)"
}

# Return value of html attribute
parse_all_attr() {
    parse_all "$1" "$2=[\"']\?\([^\"'>]*\)"
}

# Retreive "action" attribute (URL) from a <form> marker
#
# stdin: (X)HTML data (idealy, call grep_form_by_xxx before)
# stdout: result
parse_form_action() {
    parse '<[Ff][Oo][Rr][Mm]' 'action="\([^"]*\)"'
}

# Retreive "value" attribute from a named <input> marker
#
# $1: name attribute of <input> marker
# stdin: (X)HTML data
# stdout: result (can be null string if <input> has no value attribute)
parse_form_input_by_name() {
    parse "<input\([[:space:]]*[^ ]*\)*name=\"\?$1\"\?" 'value="\?\([^">]*\)' 2>/dev/null
}

# Retreive "value" attribute from a typed <input> marker
#
# $1: type attribute of <input> marker (for example: "submit")
# stdin: (X)HTML data
# stdout: result (can be null string if <input> has no value attribute)
parse_form_input_by_type() {
    parse "<input\([[:space:]]*[^ ]*\)*type=\"\?$1\"\?" 'value="\?\([^">]*\)' 2>/dev/null
}

# Return base of URL
# Example: http://www.host.com/a/b/c/d => http://www.host.com
#
# $1: URL
basename_url()
{
    # Bash >=3.0 supports regular expressions
    # [[ "$1" =~ (http://[^/]*) ]] && echo "${BASH_REMATCH[1]}" || echo "$1"
    echo $(expr match "$1" '\(http://[^/]*\)' || echo "$1")
}

# Return basename of file path
# Example: /usr/bin/foo.bar => foo.bar
#
# $1: filename
basename_file()
{
    # `basename -- "$1"` may be screwed on some BusyBox versions
    echo "${1##*/}"
}

# HTML entities will be translated
#
# stdin: data
# stdout: data (converted)
html_to_utf8() {
    if check_exec 'recode'; then
        log_report "html_to_utf8: use recode"
        recode html..utf8
    elif check_exec 'perl'; then
        log_report "html_to_utf8: use perl"
        perl -n -mHTML::Entities \
             -e 'BEGIN { eval{binmode(STDOUT,q[:utf8]);}; }; print HTML::Entities::decode_entities($_);'
    else
        log_notice "recode binary not found"
    fi
}

# Encode a text to include into an url.
# - check for "reserved characters" : $&+,/:;=?@
# - check for "unsafe characters": '<>#%{}|\^~[]`
# - check for space character
#
# stdin: data (example: relative URL)
# stdout: data (nearly complains RFC2396)
uri_encode_strict() {
    cat | sed -e 's/\$/%24/g' -e 's|/|%2F|g' -e 's/\%/%25/g' \
        -e 's/\x26/%26/g' -e 's/\x2B/%2B/g' -e 's/\x2C/%2C/g' \
        -e 's/\x3A/%3A/g' -e 's/\x3B/%3B/g' -e 's/\x3D/%3D/g' \
        -e 's/\x3F/%3F/g' -e 's/\x40/%40/g' -e 's/\x20/%20/g' \
        -e 's/\x22/%22/g' -e 's/\x3C/%3C/g' -e 's/\x3E/%3E/g' \
        -e 's/\x23/%23/g' -e 's/\x7B/%7B/g' -e 's/\x7D/%7D/g' \
        -e 's/\x7C/%7C/g' -e 's/\^/%5E/g' -e 's/\x7E/%7E/g' \
        -e 's/\x60/%60/g' -e 's/\\/%5C/g' -e 's/\[/%5B/g' -e 's/\]/%5D/g'
}

# Encode a complete url.
# - check for space character and squares brackets
# - do not check for "reserved characters" (use "uri_encode_strict" for that)
#
# Bad encoded URL request can lead to HTTP error 400.
# curl doesn't do any checks, whereas wget convert provided url.
#
# stdin: data (example: absolute URL)
# stdout: data (nearly complains RFC2396)
uri_encode() {
    cat | sed -e "s/\x20/%20/g" -e "s/\[/%5B/g" -e "s/\]/%5D/g"
}

# Decode a complete url.
# - check for space character
# - do not check for "reserved characters"
#
# stdin: data (example: absolute URL)
# stdout: data (nearly complains RFC2396)
uri_decode() {
    cat | sed -e "s/%20/\x20/g"
}

# Execute javascript code
#
# stdin: js script
# stdout: script results
# $?: boolean
javascript() {
    JS_PRG=$(detect_javascript)

    local TEMPSCRIPT=$(create_tempfile)
    cat > $TEMPSCRIPT

    $JS_PRG "$TEMPSCRIPT"
    rm -rf "$TEMPSCRIPT"
    return 0
}

# Dectect if a JavaScript interpreter is installed
#
# stdout: path of executable
# $?: boolean (0 means found)
detect_javascript() {
    if ! check_exec 'js'; then
        echo "Error: Javascript interpreter not found"
        return 1
    fi
    type -P 'js'
}

# Dectect if a Perl interpreter is installed
#
# stdout: path of executable
# $?: boolean (0 means found)
detect_perl() {
    if ! check_exec 'perl'; then
        echo "Echo: Perl interpreter not found"
        return 1
    fi
    type -P 'perl'
}

# Create a tempfile and return path
#
# $1: Suffix
create_tempfile() {
    SUFFIX=$1
    FILE="${TMPDIR:-/tmp}/$(basename_file $0).$$.$RANDOM$SUFFIX"
    : > "$FILE"
    echo "$FILE"
}

# OCR of an image.
#
# $1: optional varfile
# stdin: image (binary)
# stdout: result OCRed text
ocr() {
    # Tesseract somewhat "peculiar" arguments requirement makes impossible
    # to use pipes or process substitution. Create temporal files
    # instead (*sigh*).
    TIFF=$(create_tempfile ".tif")
    TEXT=$(create_tempfile ".txt")

    $CONVERT -compress None - tif:- > $TIFF
    $TESSERACT $TIFF ${TEXT/%.txt} 2>&1

    cat $TEXT
    rm -f $TIFF $TEXT
}

# Delete blank lines
# stdin: input (multiline) string
# stdout: result string
drop_empty_lines() {
    cat | sed '/^[    ]*$/d'
}

# Remove all temporal files created by the script
# (with create_tempfile)
remove_tempfiles() {
    rm -rf "${TMPDIR:-/tmp}/$(basename_file $0).$$.*"
}

if [ -f "$CONVERT" -a -x "$CONVERT" -a -f "$TESSERACT" -a -x "$TESSERACT" ];then
    TESSDATA_PREFIX=${utildir}/share/; export TESSDATA_PREFIX
    autocaptcha=true
else
    autocaptcha=false
fi

# List the size of the downloading/completed file
# $1: Filename
# stdout: Actual Filesize if Filename.aria2 exists otherwise Filesize
FileSize () {
    if [ -f "$1" ] ; then
        if [ -f "$1".aria2 ];then
            SIZE=$(${A2CNFO} "${1}.aria2"|tail -1)
            echo $SIZE
        else
            SIZE=$(ls -ld "$1" | awk '{print $5}')
            echo $SIZE
        fi
    else
        echo 0
    fi
}

# Update the Database
# $1: Select FIELD NAME
# $2: Select FIELD VALUE
# $3: Set FIELD NAME
# $4: Set FIELD VALUE
updateDBValue() {
    select=$1
    svalue=$2
    option=$3
    ovalue=$4
    DB_Locked=1
    while [ $DB_Locked -ne 0 ];do
        Output=$(${SQLITE} ${DBFNAME} "UPDATE links SET ${option}=\"${ovalue}\" WHERE ${select}=\"${svalue}\"" 2>&1)
        DB_Locked=$(echo $Output|grep "database is locked"|wc -l)
    done
}

# Delete a Row in DB
# $1: LINK Name
deleteDBRow() {
    url=$1
    DB_Locked=1
    while [ $DB_Locked -ne 0 ];do
        Output=$(${SQLITE} ${DBFNAME} "DELETE FROM links WHERE link=\"${url}\"" 2>&1)
        DB_Locked=$(echo $Output|grep "database is locked"|wc -l)
    done
}

# Load the recaptcha from the given URL
# $1: reCAPTCHA site public key
# stdout: image url
recaptcha_load_image() {
    URL="${RECAPTCHA_SERVER}challenge?k=${1}&ajax=1"
    log_debug "reCaptcha URL: $URL"

    VARS=$(wget "$URL" -qO-)

    if [ "$VARS" != "" ]; then
        server=$(echo "$VARS" | parse_quiet 'server' "server[[:space:]]\?:[[:space:]]\?'\([^']*\)'")
        challenge=$(echo "$VARS" | parse_quiet 'challenge' "challenge[[:space:]]\?:[[:space:]]\?'\([^']*\)'")
        log_debug "reCaptcha server: $server"
        log_debug "reCaptcha challenge: $challenge"

        echo "${server}image?c=${challenge}"
    fi
}

# $1: reCAPTCHA image filename
# stdout: challenge (string)
recaptcha_get_challenge_from_image() {
    basename "$1" | cut -d= -f2
}

# $1: reCAPTCHA site public key
# $2: reCAPTCHA image url
# stdout: new image path
recaptcha_reload_image() {
    IMAGE_URL="$2"

    if [ "$IMAGE_URL" != "" ]; then
        challenge=$(recaptcha_get_challenge_from_image "$IMAGE_URL")
        server="$RECAPTCHA_SERVER"

        STATUS=$(wget -qO- "${server}reload?k=$1&c=${challenge}&reason=r&type=image&lang=en")
        challenge=$(echo "$STATUS" | parse_quiet 'finish_reload' "('\([^']*\)")

        echo "${server}image?c=${challenge}"
    fi
}

# $1: Image URL
# stdout: TICKET_ID & reCaptcha String
submit_captcha_trader() {
    FILEURL="$1"

    if [ "$FILEURL" != "" ];then
        log_debug "$1" "Submitting to captchatrader.com."
        RESULT=$(wget --post-data="api_key=${api_key}&username=${captchauser}&password=${captchapass}&type=url-jpg&value=${FILEURL}" $CT_SUBMIT_API_URL -qO-)
        log_debug "$1" "Result :$RESULT"
        TICKET_ID=$(echo "$RESULT"|cut -d',' -f1|cut -d'[' -f2)
        CAPTCHA_STRING=$(echo "$RESULT"|cut -d'"' -f2)
        echo "$TICKET_ID:$CAPTCHA_STRING"
    fi
}

# $1: CAPTCHA is Correct or Not
# $2: TICKET_ID
respond_captcha_trader() {
    TICKET_ID="$2"
    wget --post-data="username=${captchauser}&password=${captchapass}&is_correct=${1}&ticket=${TICKET_ID}" $CT_RESPOND_API_URL -qO-
}

# stdout: CREDITS
getcredits_captcha_trader() {
    RESULT=$(wget -qO- "http://captchatrader.com/api/get_credits/username:${captchauser}/password:${captchapass}/")
    ERRCODE=$(echo "$RESULT"|cut -d',' -f1|cut -d'[' -f2)
    CREDITS=$(echo "$RESULT"|cut -d',' -f2|cut -d']' -f1)
    if [ "$ERRCODE" != "-1" ];then
        echo "$CREDITS"
    else
        echo 0
    fi
}

# $1 WAIT time
# $2 Error Message
check_router() {
    WAIT="$1"
    MESSG="$2"

    if [ $routertype -eq 1 ];then
        if [ "$rebootrouter" = "true" ];then
            log_debug "Dynamic IP. Waiting for $WAIT seconds and/or Rebooting router..." " "
            echo "Reboot:${WAIT}"
        else
            if [ "$WAIT" -ne 0 ];then
                log_debug "Dynamic IP. Waiting for $WAIT seconds..." " "
                echo "Wait:${WAIT}"
            else
                echo "Error: $MESSG"
            fi
        fi
    else
        if [ "$WAIT" -ne 0 ];then
            log_debug "Static IP. Waiting for $WAIT seconds..." " "
            echo "Wait:${WAIT}"
        else
            echo "Error: $MESSG"
        fi
    fi
}

# $1: URL
# $2: Input message
log_debug() {
    echo "$(date '+%D %X') $*" >> ${TMPDIR}/plugin_debug.log
}
